SystemOrganization addCategory: #BinaryModules!

!Morph methodsFor: '*BinaryModules' stamp: 'lw 1/13/2013 20:19'!
addStandardHaloMenuItemsTo: aMenu hand: aHandMorph
	"Add standard halo items to the menu"

	| unlockables |

	self isWorldMorph ifTrue:
		[^ self addWorldHaloMenuItemsTo: aMenu hand: aHandMorph].

	self mustBeBackmost ifFalse:
		[aMenu add: 'send to back' translated action: #goBehind.
		aMenu add: 'bring to front' translated action: #comeToFront.
		self addEmbeddingMenuItemsTo: aMenu hand: aHandMorph.
		aMenu addLine].

	self addFillStyleMenuItems: aMenu hand: aHandMorph.
	self addBorderStyleMenuItems: aMenu hand: aHandMorph.
	self addDropShadowMenuItems: aMenu hand: aHandMorph.
	self addLayoutMenuItems: aMenu hand: aHandMorph.
	self addHaloActionsTo: aMenu.
	owner isTextMorph ifTrue:[self addTextAnchorMenuItems: aMenu hand: aHandMorph].
	aMenu addLine.
	self addToggleItemsToHaloMenu: aMenu.
	aMenu addLine.
	self addCopyItemsTo: aMenu.
	self addPlayerItemsTo: aMenu.
	self addExportMenuItems: aMenu hand: aHandMorph.
	self addStackItemsTo: aMenu.
	self addMiscExtrasTo: aMenu.
	Preferences noviceMode ifFalse:
		[self addDebuggingItemsTo: aMenu hand: aHandMorph].

	aMenu addLine.
	aMenu defaultTarget: self.

	aMenu addLine.

	unlockables := self submorphs select:
		[:m | m isLocked].
	unlockables size = 1 ifTrue:
		[aMenu
			add: ('unlock "{1}"' translated format: unlockables first externalName)
			action: #unlockContents].
	unlockables size > 1 ifTrue:
		[aMenu add: 'unlock all contents' translated action: #unlockContents.
		aMenu add: 'unlock...' translated action: #unlockOneSubpart].
	
	BMBundle add: self toBinaryModuleSubmenuTo: aMenu.
	
	aMenu defaultTarget: aHandMorph.
! !

!MultiByteFileStream methodsFor: '*BinaryModules' stamp: 'lw 12/21/2012 14:52'!
nextPutAll: aCollection

	(self isBinary or: [aCollection class == ByteArray]) ifTrue: [
		^ super nextPutAll: aCollection.
	].
	aCollection class == WordArrayForSegment ifTrue: [
		^ super nextPutAll: (Bitmap withAll: aCollection) asByteArray
	].
	^converter nextPutAll: aCollection toStream: self! !

Object subclass: #BMBundle
	instanceVariableNames: 'package file name'
	classVariableNames: 'BMBundles'
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMBundle commentStamp: 'lw 1/14/2013 01:22' prior: 0!
A BMBundle is a Stub enabling to find and install BMPackages.

Instance Variables
	file:		<Object>
	name:		<Object>
	package:		<Object>

file
	Either an object which understands name, classeDescriptions and serializedObjects, or a String, which points to a file where we can find these information.
	We use the file to see that this bundle has been saved, and is not changed since then.
	Saving the filename instead of the struct, avoids saving the same information in two places in the image. In case of class-descriptions, that might not be much overhead, but they include CompiledMethod-Descriptions, with source, and serializedObjects size is unknown.

name
	A String used to find me. Should be the same as used in BMPackage and is copied to BMFile for serialization.

package
	If the bundle is installed, this variable points to the BMPackage instance, which contains the materialized classes and objects.
	If this variable is nil, the assumptions is, that it is not installed.!

!BMBundle class methodsFor: 'system integration' stamp: 'lw 1/13/2013 20:27'!
add: aMorph toBinaryModuleSubmenuTo: aMenu
	"add the possibility to add a morph to the object list of a BMPackage"
	
	| subMenu |
	subMenu := MenuMorph new defaultTarget: self.
	
	subMenu add: 'new module...' translated selector: #createNewPackageWith: argument: aMorph.
	BMBundles do: [ :each |
		each isInstalled ifTrue: [
			subMenu 
				add: each menuString 
				target: each package
				selector: #addMorph: 
				argument: aMorph]].
	
	aMenu 
		addLine;
		add: 'add to Binary Module...' translated subMenu: subMenu;
		addLine! !

!BMBundle class methodsFor: 'magic constants' stamp: 'WS 1/9/2013 17:41'!
bundleExtension
	^ '.bundle'! !

!BMBundle class methodsFor: 'system integration' stamp: 'lw 1/13/2013 20:48'!
createNewPackageWith: aMorph
	
	"Using #addMorph: instead of including the morph in the objects array, to enable adding classes to the classes array which are needed for the morph and its submorphs at only one point in BMPackage."
	^(UIManager default request: 'Please type new module name' translated) 
		ifNotNil: [ :aName | 
			((BMPackage named: aName)
				addMorph: aMorph;
				bundle) 
					addToSystem;
					yourself ]! !

!BMBundle class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 01:26'!
fromFile
	"finds only files without extension and files ending with .bundle"
	| file |
	file := UIManager default chooseFileMatching: #('' 'bundle').
	file ifNotNil: [^ super new
						fromBinary: (BMFile fromFile: file);
						addToSystem;
						yourself]! !

!BMBundle class methodsFor: 'instance creation' stamp: 'lw 1/13/2013 19:58'!
fromFile: aFileName
	^ super new
		fromBinary: (BMFile fromFile: aFileName);
		addToSystem;
		yourself! !

!BMBundle class methodsFor: 'instance creation' stamp: 'lw 1/13/2013 19:58'!
fromPackage: aPackage
	^ super new
		name: aPackage name;
		bundle: aPackage;
		addToSystem;
		yourself! !

!BMBundle class methodsFor: 'class initialization' stamp: 'lw 1/13/2013 19:39'!
initialize
	"self initialize"
	super initialize.
	BMBundles := OrderedCollection new! !

!BMBundle class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:18'!
loadNamed: aPackageName
	^ self fromFile: aPackageName, self bundleExtension! !

!BMBundle class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 01:09'!
named: aString
	
	^BMBundles 
		detect: [ :ea | ea name = aString asString ]
		ifNone: [ (UIManager default confirm: 'The bundle was not found. Do you want to create it?' translated) ifFalse: [^nil].
			(BMPackage named: aString asString) bundle
				addToSystem;
				yourself]! !

!BMBundle class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:01'!
new
	self shouldNotImplement! !

!BMBundle methodsFor: 'package facade' stamp: 'lw 1/14/2013 00:59'!
addClass: aClass
	^self package addClass: aClass! !

!BMBundle methodsFor: 'package facade' stamp: 'lw 1/14/2013 01:03'!
addObject: anObject
	^self package addObject: anObject    ! !

!BMBundle methodsFor: 'utilities' stamp: 'lw 1/13/2013 19:43'!
addToSystem
	(BMBundles ifNil: [BMBundles := OrderedCollection new])
		addLast: self! !

!BMBundle methodsFor: 'creation' stamp: 'lw 1/13/2013 19:42'!
bundle: aPackage
	self assert: name = aPackage name.
	package := aPackage.
	package addDependent: self! !

!BMBundle methodsFor: 'utilities' stamp: 'WS 1/9/2013 18:00'!
createClassDescriptionsFor: aCollection
	^ aCollection collect: [ :class | BMClassDescription for: class ]! !

!BMBundle methodsFor: 'installation' stamp: 'lw 1/13/2013 18:49'!
createClasses
	self assert: file notNil.
	^ self file classDescriptions collect: [ :classDescription | classDescription install ]! !

!BMBundle methodsFor: 'saving' stamp: 'WS 1/9/2013 17:41'!
exportToFile
	^ self exportToFile: name, self class bundleExtension! !

!BMBundle methodsFor: 'saving' stamp: 'lw 1/13/2013 19:54'!
exportToFile: aFileName
	FLSerializer serialize: self file toFileNamed: aFileName.
	file := aFileName! !

!BMBundle methodsFor: 'saving' stamp: 'lw 1/14/2013 01:15'!
extractToFile: aFileName
	self exportToFile: aFileName.
	self uninstall! !

!BMBundle methodsFor: 'accessing-derived' stamp: 'lw 1/13/2013 19:56'!
file
	"do we need to save the created file?"
	^file 
		ifNil: [BMFile 
				named: name 
				classDescriptions: (self createClassDescriptionsFor: self package classes) 
				serializedObjects: (FLSerializer serializeToByteArray: self package objects) ]
		ifNotNil: [file isSymbol | file isString 
					ifTrue: [BMFile fromFile: file]
					ifFalse: [file]]! !

!BMBundle methodsFor: 'creation' stamp: 'lw 1/13/2013 18:46'!
fromBinary: aBMFile
	name := aBMFile name.
	file := aBMFile! !

!BMBundle methodsFor: 'installation' stamp: 'lw 1/13/2013 18:55'!
install
	self assert: file notNil.
	package := BMPackage
		named: name
		withClasses: self createClasses
		andObjects: self objects.
	package addDependent: self.
	^package! !

!BMBundle methodsFor: 'testing' stamp: 'lw 1/13/2013 20:27'!
isInstalled
	^package notNil! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:40'!
name
	^ name! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:40'!
name: aString
	name := aString! !

!BMBundle methodsFor: 'installation' stamp: 'lw 1/13/2013 18:50'!
objects
	self assert: file notNil.
	^ FLMaterializer materializeFromByteArray: file serializedObjects! !

!BMBundle methodsFor: 'accessing-derived' stamp: 'lw 1/13/2013 18:40'!
package
	^package ifNil: [self notify: 'package is not installed.' ]! !

!BMBundle methodsFor: 'utilities' stamp: 'lw 1/14/2013 01:12'!
removeFromSystem
	BMBundles ifNotNil: 
		[BMBundles := BMBundles reject: [ :each | each == self]]! !

!BMBundle methodsFor: 'accessing-derived' stamp: 'lw 1/13/2013 18:42'!
serializedObjects
	^file 
		ifNil: [self error: 'Bundle not created from file.']
		ifNotNil: [ file serializedObjects ]  ! !

!BMBundle methodsFor: 'installation' stamp: 'lw 1/13/2013 21:28'!
uninstall
	"although unload depends heavily on data of Package, for symmetrie to install, it is implemented here"
	self isInstalled 
		ifTrue: [ 
			file ifNil: [self notify: 'Are you sure? The classes and objects will be irrevocably removed.'].
			self package classes do: [ :aClass | aClass removeFromSystem].
			"Only for morphs, we know how to remove them."
			self package objects do: [ :each | each isMorph ifTrue: [each delete]].
			package := nil]! !

Object subclass: #BMClassDescription
	instanceVariableNames: 'name instanceVariables category compiledMethods superclass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMClassDescription class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:08'!
for: aClass
	^ super new
		initializeFrom: aClass;
		yourself! !

!BMClassDescription class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:08'!
new
	^ self shouldNotImplement! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'dg 1/10/2013 12:26'!
initializeFrom: aClass
	
	name := aClass name.
	superclass := aClass superclass name.
	instanceVariables := aClass instVarNames.
	category := aClass category.
	compiledMethods := aClass methodDict collect: [ :each | BMCompiledMethod for: each].! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'dg 1/10/2013 14:40'!
install
	| newClass |
	
	(Smalltalk at: name ifAbsent: nil) ifNotNil: [self error: 'Class already exists: ', name asString ].
	newClass := (Smalltalk at: superclass) subclass: name
		instanceVariableNames: (String streamContents: [ :stream | instanceVariables do: [ :each | stream << each <<$ ] ])
		classVariableNames: ''
		poolDictionaries: ''
		category: category.
		
	compiledMethods do: [ :aMethod | aMethod addTo: newClass].
	
	^newClass! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'dg 1/10/2013 10:50'!
name
	^name! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'dg 1/10/2013 11:23'!
represents: aClass

	instanceVariables = aClass instVarNames ifFalse: [^false].
	category = aClass category ifFalse: [^false].
	compiledMethods size = aClass methodDict size ifFalse: [^false].
	^true! !

Object subclass: #BMCompiledMethod
	instanceVariableNames: 'bytecode protocol source selector headerWord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMCompiledMethod class methodsFor: 'instance creation' stamp: 'dg 1/10/2013 11:11'!
for: aMethod

	^super new
			initializeFrom: aMethod; 
			yourself! !

!BMCompiledMethod class methodsFor: 'instance creation' stamp: 'dg 1/10/2013 11:09'!
new

	^self shouldNotImplement! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'lw 1/10/2013 16:36'!
addTo: aClass

	| aCompiledMethod |
	aCompiledMethod := CompiledMethod newMethod: (bytecode size - ((self numLiterals + 1) * 4)) 
											header: headerWord.
	aCompiledMethod replaceFrom: 1 to: bytecode size with: bytecode.
	"If interrupted at this point the VM might very likely crash."
	aCompiledMethod methodClass: aClass.
	aClass methodDict at: selector put: aCompiledMethod.
	
	"aCompiledMethod properties at: #source put: source"! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'dg 1/10/2013 15:03'!
initializeFrom: aMethod

	protocol := aMethod methodClass organization categoryOfElement: aMethod selector.
	selector := aMethod selector.
	source := aMethod getSource.
	bytecode := ByteArray newFrom: aMethod.
	headerWord := aMethod header! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'lw 1/10/2013 16:35'!
numLiterals
	"Answer the number of literals used by the receiver."
	"copy from CompiledMethod"
	^ (headerWord bitShift: -9) bitAnd: 16rFF! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'lw 1/10/2013 16:23'!
printOn: aStream

	super printOn: aStream.
	aStream << '(#' << selector << ', ' << headerWord << ', ' << bytecode size << ')'! !

Object subclass: #BMFile
	instanceVariableNames: 'classDescriptions serializedObjects name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMFile class methodsFor: 'instance creation' stamp: 'lw 1/13/2013 18:25'!
fromFile: aFileName
	| bundle |
	bundle := FLMaterializer materializeFromFileNamed: aFileName.
	^ (bundle isKindOf: self)
		ifTrue: [ bundle ]
		ifFalse: [ nil ]! !

!BMFile class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 00:50'!
named: aName classDescriptions: someBMClassDescriptions serializedObjects: someBinary
	^self new
		name: aName asString;
		classDescriptions: someBMClassDescriptions;
		serializedObjects: someBinary;
		yourself      ! !

!BMFile methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:42'!
classDescriptions
	^ classDescriptions! !

!BMFile methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:43'!
classDescriptions: aCollection
	classDescriptions := aCollection! !

!BMFile methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:40'!
name
	^ name! !

!BMFile methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:40'!
name: aString
	name := aString! !

!BMFile methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:42'!
serializedObjects
	^ serializedObjects! !

!BMFile methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:43'!
serializedObjects: aByteArray
	serializedObjects := aByteArray! !

Object subclass: #BMPackage
	instanceVariableNames: 'name classes objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMPackage commentStamp: 'lw 1/14/2013 01:23' prior: 0!
A BMPackage is the installed version of a BMBundle. It contains an Array of all Classes belonging to this Bundle.

Instance Variables
	classes:		<Object>
	name:		<Object>
	objects:		<Object>

classes
	- xxxxx

name
	- xxxxx

objects
	- xxxxx
!

!BMPackage class methodsFor: 'as yet unclassified' stamp: 'lw 1/13/2013 17:39'!
installFromFileNamed: aFileName
	^ (BMBundle fromFileNamed: aFileName)
		ifNil: [ self error: 'Error during bundle load' ]
		ifNotNil: [ :bundle | bundle install ]! !

!BMPackage class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 01:10'!
named: name
	^ super new
		name: name asString;
		classes: #();
		objects: #();
		yourself! !

!BMPackage class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 00:50'!
named: aString withClasses: classes andObjects: objects
	^ (self named: aString asString)
		classes: classes;
		objects: objects;
		yourself! !

!BMPackage class methodsFor: 'instance creation' stamp: 'RL 1/9/2013 11:18'!
new
	self shouldNotImplement! !

!BMPackage class methodsFor: 'as yet unclassified' stamp: 'WS 1/9/2013 18:19'!
removeFromSystem
	^ self notYetImplemented! !

!BMPackage methodsFor: 'accessing - derived' stamp: 'lw 1/14/2013 01:06'!
addClass: aClass
	
	self assert: aClass isMeta not.
	(classes includes: aClass) ifFalse: 
		[classes := classes copyWith: aClass].
	^aClass! !

!BMPackage methodsFor: 'system integration' stamp: 'lw 1/14/2013 01:02'!
addMorph: aMorph
	
	self notify: 'Remember to also add the classes neccessary to create the morph.'.
	self addObject: aMorph  ! !

!BMPackage methodsFor: 'accessing - derived' stamp: 'lw 1/14/2013 01:08'!
addObject: anObject
	
	(objects includes: anObject) ifFalse:
		[objects := objects copyWith: anObject].
	^anObject! !

!BMPackage methodsFor: 'as yet unclassified' stamp: 'lw 1/13/2013 17:36'!
bundle
	^ BMBundle fromPackage: self! !

!BMPackage methodsFor: 'accessing' stamp: 'WS 1/9/2013 18:02'!
classes
	^ classes! !

!BMPackage methodsFor: 'accessing' stamp: 'lw 1/13/2013 18:18'!
classes: aCollection
	classes := aCollection.
	self changed: #classes: with: aCollection! !

!BMPackage methodsFor: 'bundle behavior' stamp: 'WS 1/9/2013 18:05'!
exportToFile
	^ self bundle exportToFile! !

!BMPackage methodsFor: 'bundle behavior' stamp: 'WS 1/9/2013 18:05'!
exportToFile: aFileName
	^ self bundle exportToFile: aFileName! !

!BMPackage methodsFor: 'bundle behavior' stamp: 'lw 1/13/2013 19:45'!
extractToFile: aFileName
	^ self bundle extractToFile: aFileName! !

!BMPackage methodsFor: 'accessing' stamp: 'RL 1/9/2013 11:21'!
name
	^ name! !

!BMPackage methodsFor: 'accessing' stamp: 'lw 1/13/2013 18:18'!
name: aName
	name := aName.
	self changed: #name: with: aName.! !

!BMPackage methodsFor: 'accessing' stamp: 'WS 1/9/2013 18:03'!
objects
	^ objects! !

!BMPackage methodsFor: 'accessing' stamp: 'lw 1/13/2013 18:19'!
objects: aCollection
	objects := aCollection.
	self changed: #objects: with: aCollection! !

Object subclass: #ExampleProject
	instanceVariableNames: 'foo bar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!ExampleProject methodsFor: 'as yet unclassified' stamp: 'RL 1/7/2013 16:29'!
foobar
	^ foo + bar! !

!ExampleProject methodsFor: 'initialization' stamp: 'RL 1/7/2013 16:29'!
initialize
	foo := 42.
	bar := 23.! !

TestCase subclass: #BMPackageTest
	instanceVariableNames: 'class bundle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMPackageTest methodsFor: 'utility' stamp: 'lw 1/13/2013 22:58'!
compareClass: aClass with: anotherClass

	self 
		assert: aClass name == anotherClass name;
		assert: aClass methodDict size equals: anotherClass methodDict size;
		assert: aClass instVarNames equals: anotherClass instVarNames;
	 	assert: aClass category equals: anotherClass category;
	 	assert: aClass superclass == anotherClass superclass.
		
	aClass methodDict associationsDo: [ :assoc | self assert: (anotherClass methodDict at: assoc key) size 
														equals: assoc value size].
	self assert: aClass class methodDict size equals: anotherClass class methodDict size.
	aClass class methodDict associationsDo: [ :assoc | self assert: (anotherClass class methodDict at: assoc key) size 
														equals: assoc value size]! !

!BMPackageTest methodsFor: 'test graph' stamp: 'lw 1/13/2013 22:39'!
setupComplexTestClass
	
	class := Object subclass: #BMTestClass
		instanceVariableNames: 'lorem'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'BinaryModules'.
		
	class compile: 'ipsum ^true'.
	class compile: 'dolor ^false'.
	class class compile: 'sit: amet amet isNil. ^Object new'.

	^ class! !

!BMPackageTest methodsFor: 'test graph' stamp: 'lw 1/10/2013 17:25'!
setupTestClass
	
	class := Object subclass: #BMTestClass
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'BinaryModules'.
		
	class compile: 'foo ^true'.
	class compile: 'bar ^false'.

	^ class! !

!BMPackageTest methodsFor: 'test graph' stamp: 'lw 1/13/2013 22:47'!
simpleTestClass
	
	class ifNotNil: [^class].
	class := Object subclass: #BMSimpleTestClass
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'BinaryModules'.
	
	^ class! !

!BMPackageTest methodsFor: 'running' stamp: 'lw 1/13/2013 19:35'!
tearDown

	super tearDown.
	class ifNotNil: [class removeFromSystem].
	bundle ifNotNil: [bundle removeFromSystem]! !

!BMPackageTest methodsFor: 'utility' stamp: 'lw 1/10/2013 17:21'!
tearDownClass: testClass
	"testClass superclass removeSubclass: testClass"
	
	testClass removeFromSystem! !

!BMPackageTest methodsFor: 'testing' stamp: 'dg 1/10/2013 10:54'!
testClassDescriptionCreation

	|aDescription|

	aDescription := BMClassDescription for: ExampleProject.
	self assert: aDescription name equals: #ExampleProject.
	self assert: (aDescription represents: ExampleProject).
! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/10/2013 17:27'!
testCreatePackage
	
	| aPackage |
	self setupTestClass.
	aPackage := BMPackage 
						named: 'testPackage' 
						withClasses: {class} 
						andObjects: {class new}.
	self assert: aPackage classes first equals: class.
	self assert: aPackage objects first class equals: class! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/13/2013 20:02'!
testDeserializePackage
	
	| aClassCopy  aPackage |
	self setupTestClass.
	((BMPackage 
			named: 'testPackage' 
			withClasses: {class} 
			andObjects: {class new}) 
		exportToFile: 'testPackage') 
			removeFromSystem.
	aClassCopy := class copy.
	class removeFromSystem.
	bundle := BMBundle fromFile: 'testPackage'.
	self 
		should: [FLMaterializer materializeFromByteArray: bundle serializedObjects] 
		raise: FLClassNotFound.
	aPackage := bundle install.
	self compareClass: aClassCopy with: (Smalltalk at: aPackage classes first name).
	self assert: aPackage objects first class equals: (Smalltalk at: aPackage classes first name).! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/13/2013 21:07'!
testMaterializeClassDescription
	class := self setupTestClass.
	self testMaterializeSimpleClassDescription! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/13/2013 22:57'!
testMaterializeComplexClassDescription
	"self new testMaterializeComplexClassDescription"
	class := self setupComplexTestClass.
	self testMaterializeSimpleClassDescription! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/13/2013 22:44'!
testMaterializeSimpleClassDescription
	" destroys the content of the instance variable class "
	| aCopy aDescription aMaterializedClass |
	[aCopy := self simpleTestClass copy. "To avoid comparing the class to itself."
	aDescription := BMClassDescription for: class.
	(Smalltalk at: class name ifAbsent: nil) ifNotNil: [ :obsoleteClass | obsoleteClass removeFromSystem].
	aMaterializedClass := aDescription install.
	
	self compareClass: aCopy with: aMaterializedClass.
	self deny: aCopy == aMaterializedClass.]
		ensure: [aMaterializedClass ifNotNil: [
					self tearDownClass: aMaterializedClass]].! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/13/2013 20:02'!
testSerializePackage
	
	| aClass |
	aClass := self setupTestClass.
	bundle := (BMPackage 
			named: 'testPackage' 
			withClasses: {aClass} 
			andObjects: {aClass new}) 
		exportToFile: 'testPackage'.
	self assert: (FileDirectory default fileExists: 'testPackage').
	FileDirectory default deleteFileNamed: 'testPackage'.
	self deny: (FileDirectory default fileExists: 'testPackage').
	aClass removeFromSystem! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/13/2013 21:28'!
testUninstallPackage
	
	|  |
	self setupTestClass.
	bundle := (BMPackage 
				named: 'testPackage' 
				withClasses: {class} 
				andObjects: {class new}) bundle.
	self assert: bundle isInstalled.
	self assert: class allInstances size equals: 1.
	[bundle uninstall] on: Warning do: [ :warning | warning resume ].
	Smalltalk garbageCollect.
	self deny: bundle isInstalled.
	self assert: class isObsolete.
	self assert: class allInstances size equals: 0.! !

ImageSegment subclass: #BMImageSegment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMImageSegment class methodsFor: 'as yet unclassified' stamp: 'lw 1/13/2013 21:34'!
install
	! !

!BMImageSegment methodsFor: 'as yet unclassified' stamp: 'lw 1/14/2013 01:13'!
install
	
	[^super install] on: Exception do: [self halt]! !
BMBundle initialize!
